@@ -30,12 +30,14 @@
 #include <gpe/gpeclockface.h>
 #include <gpe/schedule.h>
 
-GtkWidget *panel_window, *time_label;
+GtkWidget *panel_window, *time_label, *face;
+GtkObject *hour_adj, *minute_adj;
 
 #define _(x) gettext(x)
 
 static gboolean show_seconds;
 static gboolean format_24 = TRUE;
+static gboolean flag_graphical = FALSE;
 
 static gchar *alarm_file, *prefs_file;
 
@@ -186,6 +188,7 @@ extract_time (struct alarm_state *alarm)
 {
   struct tm tm;
   time_t t;
+  int wasdst;
 
   memset (&tm, 0, sizeof (tm));
 
@@ -197,7 +200,11 @@ extract_time (struct alarm_state *alarm)
 
   t = mktime (&tm);
 
+  wasdst = tm.tm_isdst;
+
   /* Break it out again to get DST right */
+  memset (&tm, 0, sizeof (tm));
+
   localtime_r (&t, &tm);
 
   tm.tm_year = alarm->year - 1900;
@@ -205,6 +212,7 @@ extract_time (struct alarm_state *alarm)
   tm.tm_mday = alarm->day;
   tm.tm_hour = alarm->hour;
   tm.tm_min = alarm->minute;
+  tm.tm_isdst = wasdst;
 
   return mktime (&tm);
 }
@@ -267,7 +275,7 @@ set_alarm (struct alarm_state *alarm)
 	  alarm->month = tm.tm_mon;
 	  alarm->day = tm.tm_mday;
 	}
-      
+
       if (schedule_set_alarm (1, t, "gpe-announce\ngpe-clock --check-alarm\n") == FALSE)
 	{
 	  gpe_error_box (_("Unable to set alarm"));
@@ -277,7 +285,7 @@ set_alarm (struct alarm_state *alarm)
 }
 
 static void
-update_time (GtkWidget *label)
+update_time_label (GtkWidget *label)
 {
   char buf[256];
   time_t t;
@@ -300,11 +308,33 @@ update_time (GtkWidget *label)
   gtk_label_set_text (GTK_LABEL (label), buf);
 }
 
+static void
+update_time_face (void)
+{
+  time_t t;
+  struct tm tm;
+
+  time (&t);
+  localtime_r (&t, &tm);
+
+  gtk_adjustment_set_value (GTK_ADJUSTMENT (hour_adj), tm.tm_hour);
+  gtk_adjustment_set_value (GTK_ADJUSTMENT (minute_adj), tm.tm_min);
+}
+
+static void
+update_time (void)
+{
+  if (flag_graphical)
+    update_time_face ();
+  else
+    update_time_label (time_label);
+}
+
 static void
 set_seconds (GtkWidget *w, GtkWidget *time_label)
 {
   show_seconds = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (w));
-  update_time (time_label);
+  update_time ();
   flush_prefs ();
 }
 
@@ -312,7 +342,7 @@ static void
 set_format (GtkWidget *w, GtkWidget *time_label)
 {
   format_24 = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (w));
-  update_time (time_label);
+  update_time ();
   flush_prefs ();
 }
 
@@ -472,8 +502,12 @@ alarm_window (void)
 				  GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
   gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (scrolled_window), GTK_SHADOW_NONE);
 
+#if 0
   gtk_box_pack_start (GTK_BOX (GTK_DIALOG (ctx->window)->vbox), scrolled_window, TRUE, TRUE, 0);
   gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW (scrolled_window), scrolled_vbox);
+#else
+  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (ctx->window)->vbox), scrolled_vbox, TRUE, TRUE, 0);
+#endif
 
   ctx->date_button = gtk_radio_button_new_with_label (NULL, _("Date:"));
   radiogroup = gtk_radio_button_group (GTK_RADIO_BUTTON (ctx->date_button));
@@ -539,10 +573,12 @@ alarm_window (void)
   button_toggled (NULL, ctx);
 }
 
-static void
+static gboolean
 clicked (GtkWidget *w, GdkEventButton *ev, GtkWidget *menu)
 {
   gtk_menu_popup (GTK_MENU (menu), NULL, NULL, gpe_popup_menu_position, w, ev->button, ev->time);
+
+  return TRUE;
 }
 
 static void
@@ -570,6 +606,30 @@ main (int argc, char *argv[])
   bind_textdomain_codeset (PACKAGE, "UTF-8");
   textdomain (PACKAGE);
 
+  while (1)
+    {
+      int this_option_optind = optind ? optind : 1;
+      int option_index = 0;
+      int c;
+
+      static struct option long_options[] = {
+	{"graphical", 0, 0, 'g'},
+	{0, 0, 0, 0}
+      };
+      
+      c = getopt_long (argc, argv, "d",
+		       long_options, &option_index);
+      if (c == -1)
+	break;
+
+      switch (c)
+	{
+	case 'g':
+	  flag_graphical = TRUE;
+	  break;
+	}
+    }
+
   alarm_file = g_strdup_printf ("%s/.gpe/alarm", g_get_home_dir ());
   prefs_file = g_strdup_printf ("%s/.gpe/clock_prefs", g_get_home_dir ());
 
@@ -585,20 +645,6 @@ main (int argc, char *argv[])
   set_defaults ();
   load_prefs ();
 
-  panel_window = gtk_plug_new (0);
-
-  time_label = gtk_label_new (NULL);
-
-  update_time (time_label);
-
-  gtk_container_add (GTK_CONTAINER (panel_window), time_label);
-
-  gtk_widget_realize (panel_window);
-
-  gtk_widget_show_all (panel_window);
-
-  gpe_system_tray_dock (panel_window->window);
-
   menu = gtk_menu_new ();
   menu_prefs = gtk_menu_item_new_with_label (_("Preferences"));
   menu_set_time = gtk_menu_item_new_with_label (_("Set the time"));
@@ -620,13 +666,47 @@ main (int argc, char *argv[])
   g_signal_connect (G_OBJECT (menu_alarm), "activate", G_CALLBACK (alarm_window), NULL);
   g_signal_connect (G_OBJECT (menu_remove), "activate", G_CALLBACK (gtk_main_quit), NULL);
 
-  g_signal_connect (G_OBJECT (panel_window), "button-press-event", G_CALLBACK (clicked), menu);
-  gtk_widget_add_events (panel_window, GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK);
+  panel_window = gtk_plug_new (0);
+
+  if (flag_graphical)
+    {
+      hour_adj = gtk_adjustment_new (0, 0, 23, 1, 15, 15);
+      minute_adj = gtk_adjustment_new (0, 0, 59, 1, 15, 15);
+  
+      face = gpe_clock_face_new (GTK_ADJUSTMENT (hour_adj), GTK_ADJUSTMENT (minute_adj), NULL);
+      gpe_clock_face_set_do_grabs (GPE_CLOCK_FACE (face), FALSE);
+      gpe_clock_face_set_radius (GPE_CLOCK_FACE (face), 16);
+      gpe_clock_face_set_label_hours (GPE_CLOCK_FACE (face), FALSE);
+      gpe_clock_face_set_hand_width (GPE_CLOCK_FACE (face), 1.0);
+      
+      gtk_container_add (GTK_CONTAINER (panel_window), face);
+
+      g_signal_connect (G_OBJECT (face), "button-press-event", G_CALLBACK (clicked), menu);
+    }
+  else
+    {
+      time_label = gtk_label_new (NULL);
+
+      update_time_label (time_label);
+
+      gtk_container_add (GTK_CONTAINER (panel_window), time_label);
+
+      g_signal_connect (G_OBJECT (panel_window), "button-press-event", G_CALLBACK (clicked), menu);
+      gtk_widget_add_events (panel_window, GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK);
+    }
+
+  gtk_widget_realize (panel_window);
+
+  gtk_widget_show_all (panel_window);
+
+  printf("allocation: %d %d\n", panel_window->allocation.width, panel_window->allocation.height);
+
+  gpe_system_tray_dock (panel_window->window);
 
   tooltips = gtk_tooltips_new ();
   gtk_tooltips_set_tip (GTK_TOOLTIPS (tooltips), panel_window, _("This is the clock.\nTap here to set the alarm, set the time, change the display format, or remove this program from the panel."), NULL);
 
-  g_timeout_add (1000, (GtkFunction) update_time, time_label);
+  g_timeout_add (1000, (GtkFunction) update_time, NULL);
 
   gtk_main ();
 
